function example_half_wave_control
% example_half_wave_control - Example for Flickermeter Simulator
%
% In this example the impact of different load switching methods is examined.
% The impaired line voltage signal is generated for the following
% scenarios:
% - hard switching
% - soft switching using half wave control
% In all cases the same load is used and periodically switched on and off.
%
% Requires MATLAB with Signal Processing Toolbox installed or Octave.
% For more information refer to [1].
%===============================================================================
% References:
% [1] https://www.solcept.ch/en/blog/complex-systems/flickersim/
%===============================================================================
%  (c) Copyright 2021 Solcept AG
%  Distributed under the Boost Software License, Version 1.0. (See accompanying
%  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
%===============================================================================

clear variables

%% Configuration

U_LINE = 230;  % line voltage in Volts
F_LINE = 50;   % line frequency in Hz
FS    = 2000;  % sampling frequency in Hz
POWER = 1400;  % load in W

SWITCHING_INTERVAL = 3;  % in sec
NUMOF_CYCLES       = 10; % number of switching cycles to simulate

SWITCHING_SEQUENCE_ON  = [0, 1, 0, 0, 1, 0, 1, 1, 0, 1];
SWITCHING_SEQUENCE_OFF = [1, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 0, 1];
% SWITCHING_SEQUENCE_ON  = [1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, ...
%                           1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0];
% SWITCHING_SEQUENCE_OFF = [1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, ...
%                           1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0];

FIGURE_TAG = 'example_half_wave_control';

%% Preparations

% one switching cycle consists of an OFF level phase, a rising ramp, an ON
% level phase and a falling ramp
cycle_ctrl = [...
  zeros(1, SWITCHING_INTERVAL * FS), ...
  ones(1, SWITCHING_INTERVAL * FS)];
ctrl = repmat(cycle_ctrl, 1, NUMOF_CYCLES);

%% Get or create browser figure handle
h_figure = findobj(0, 'Tag', FIGURE_TAG);
if (isempty(h_figure))
  h_figure = figure;
  set(h_figure, 'Tag', FIGURE_TAG);
end
figure(h_figure);
clf(h_figure);

%% Hard switching

p = (ctrl > 0.5) * POWER;
[u, fs, du] = power_to_line_voltage(p, FS, U_LINE, F_LINE);

Pst = flicker_sim(u, fs, F_LINE, U_LINE);

figure(h_figure)
subplot(2, 1, 1, 'parent', h_figure);
t = [0 : length(du) - 1] / fs;
plot(t, du);
set(gca, 'xlim', [0, 2 * SWITCHING_INTERVAL]);
grid on
title(sprintf('Example: No Power Pulse Shaping (Hard Switching)\nPst = %.2f', Pst))
xlabel('Time [sec]')
ylabel('Voltage drop [V]')

%% Half wave control

numof_half_waves = (2 * SWITCHING_INTERVAL * 2 * F_LINE - length(SWITCHING_SEQUENCE_ON) - length(SWITCHING_SEQUENCE_OFF));
numof_off_half_waves = floor(numof_half_waves / 2);
numof_on_half_waves = ceil(numof_half_waves - numof_off_half_waves);

SWITCHING_SEQUENCE = [...
  zeros(1, numof_off_half_waves), ...
  SWITCHING_SEQUENCE_ON, ...
  ones(1, numof_on_half_waves), ...
  SWITCHING_SEQUENCE_OFF ...
  ];

repeated_switching_sequence = repmat(SWITCHING_SEQUENCE, 1, NUMOF_CYCLES);
repeated_switching_sequence = repeated_switching_sequence(:);

% generate half wave signal with and without load
[u_off, fs, du] = power_to_line_voltage(zeros(1, (FS / F_LINE) / 2), FS, U_LINE, F_LINE);
[u_on, fs, du] = power_to_line_voltage(POWER * ones(1, (FS / F_LINE) / 2), FS, U_LINE, F_LINE);

% generate line signal
numof_half_waves = length(repeated_switching_sequence);
samples_per_half_wave = length(u_off);
u = zeros(1, numof_half_waves * samples_per_half_wave);
u_wo_load = zeros(1, numof_half_waves * samples_per_half_wave);

u_idx = 1;
for (idx = 1 : numof_half_waves)
  % determine sign for next have wave
  if (mod(idx, 2) == 1)
    wave_sign = 1;
  else
    wave_sign = -1;
  end
  
  % add half wave
  if (repeated_switching_sequence(idx) == 1)
    u(u_idx : u_idx + samples_per_half_wave - 1) = wave_sign * u_on;
  else
    u(u_idx : u_idx + samples_per_half_wave - 1) = wave_sign * u_off;
  end
  u_wo_load(u_idx : u_idx + samples_per_half_wave - 1) = wave_sign * u_off;
  u_idx = u_idx + samples_per_half_wave;
end

Pst = flicker_sim(u, FS, F_LINE, U_LINE);

figure(h_figure)
subplot(2, 1, 2, 'parent', h_figure);
du = u_wo_load - u;
t = [0 : length(du) - 1] / fs;
plot(t, du);
grid on
set(gca, 'xlim', [0, 2 * SWITCHING_INTERVAL]);
title(sprintf('Example: Power Pulse Shaping using Have Wave Control\nPst = %.2f', Pst))
xlabel('Time [sec]')
ylabel('Voltage drop [V]')
