function example_testbench_120V_sin
% example_testbench_50Hz - Flickermeter Simulator Performance Test
%
% Performs the performance tests according [1], section 6.2 for a
% 120V system.
%
% Requires MATLAB with Signal Processing Toolbox installed or Octave.
% For more information refer to [2].
%===============================================================================
% References:
% [1] IEC 61000-4-15, Electromagnetic compatibility (EMC), Testing and
%     measurement techniques, Flickermeter, Edition 2.0, 2010-08
% [2] https://www.solcept.ch/en/blog/complex-systems/flickersim/
%===============================================================================
%  (c) Copyright 2023 Solcept AG
%  Distributed under the Boost Software License, Version 1.0. (See accompanying
%  file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
%===============================================================================

clear variables

%% Configuration

V_NOM                =  120;  % line voltage in V
FS                   = 4000;  % sampling frequency in Hz
OBSERVATION_INTERVAL =  600;  % duration of simulation in sec
ERROR_LIMIT          =    8;  % maximum deviation of Pst in percent

IS_OCTAVE = exist('OCTAVE_VERSION') ~= 0;

%% Definition of test cases (according [1], section 5.1 table 1a)

% voltage changes per minute
fluc_freq = [0.5, 1, 1.5, 2, 2.5, 3, 3.5, 4, 4.5, 5, 5.5, 6, 6.5, 7, 7.5, 8, 8.8, 9.5, ...
    10, 10.5, 11, 11.5, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25, 33+1/3, 40];

% magnitude of relative voltage changes in percent (60 Hz contain one more test)
d = [2.453, 1.465, 1.126, 0.942, 0.815, 0.717, 0.637, 0.570, 0.514, 0.466, 0.426, 0.393, 0.366, 0.346, 0.332, 0.323, 0.321, 0.329, ...
    0.341, 0.355, 0.373, 0.394, 0.417, 0.469, 0.528, 0.592, 0.660, 0.734, 0.811, 0.892, 0.977, 1.067, 1.160, 1.257, 1.359, 1.464, 2.570, 4.393;   % 60 Hz system

    2.453, 1.465, 1.126, 0.942, 0.815, 0.717, 0.637, 0.570, 0.514, 0.466, 0.426, 0.393, 0.366, 0.346, 0.332, 0.323, 0.321, 0.329, ...
    0.341, 0.355, 0.373, 0.394, 0.417, 0.469, 0.528, 0.592, 0.660, 0.734, 0.811, 0.892, 0.978, 1.068, 1.162, 1.261, 1.365, 1.476, 3.111, 0.000];  % 50 Hz system

% line frequency in Hz
f_nom = [60, 50];

%% Run Tests
fprintf('\nStarting Flicker Model Test\n\n');

overall_max_err = 0;
all_tests_passed = 1;
t = 0 : 1 / FS : OBSERVATION_INTERVAL;

for (freq_idx = 1:size(d, 1))
    fprintf('Testing with frequency: %3d \n\n', f_nom(freq_idx));
    for (idx = 1 : length(fluc_freq))
        if (d(freq_idx, idx) > 0) % Skip 40 Hz fluctuation test for 50 Hz system
            a_mod = d(freq_idx, idx) / 100;

            % create test signal (according [1], Annex B)
            u = V_NOM * sin(2 * pi * f_nom(freq_idx) * t) .* (1 + a_mod * 1 / 2 .* sin(2 * pi * fluc_freq(idx) * t));

            [P_st(idx), P_inst] = flicker_sim(u, FS, f_nom(freq_idx), V_NOM);
            err(idx) = (max(P_inst) - 1) * 100;

            fprintf('V_nominal = %2d V, F_LINE = %2d Hz, freq = %5.1f Hz, d = %6.3f %%:  Pinst,max = %1.3f, err = %4.1f %%\n', ...
                V_NOM, f_nom(freq_idx), fluc_freq(idx), a_mod * 100, max(P_inst), err(idx))
            if (IS_OCTAVE)
                fflush(stdout);
            end
        end
    end

    max_err = max(abs(err));
    if (max_err <= ERROR_LIMIT)
        fprintf('Test passed (max. error: %.1f %%)\n\n', max_err);
    else
        fprintf('Test failed (max. error: %.1f %%)\n\n', max_err);
        all_tests_passed = 0;
    end
    if max_err > overall_max_err
        overall_max_err = max_err;
    end
    err = 0;
end

fprintf('Overall maximum error: %.1f\n', overall_max_err);
if (all_tests_passed)
    fprintf('ALL TESTS PASSED\n');
end